@@ -10,9 +10,10 @@ import (
 	"time"
 
 	"k8s.io/apimachinery/pkg/util/intstr"
+	"k8s.io/klog/v2"
 
-	"github.com/golang/glog"
 	mapiv1 "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
+	"github.com/openshift/machine-api-operator/pkg/metrics"
 	"github.com/openshift/machine-api-operator/pkg/util/conditions"
 	corev1 "k8s.io/api/core/v1"
 	apimachineryerrors "k8s.io/apimachinery/pkg/api/errors"
@@ -33,14 +34,13 @@ import (
 const (
 	machineAnnotationKey          = "machine.openshift.io/machine"
 	machineExternalAnnotationKey  = "host.metal3.io/external-remediation"
-	ownerControllerKind           = "MachineSet"
 	nodeMasterLabel               = "node-role.kubernetes.io/master"
 	machineRoleLabel              = "machine.openshift.io/cluster-api-machine-role"
 	machineMasterRole             = "master"
 	machinePhaseFailed            = "Failed"
 	remediationStrategyAnnotation = "machine.openshift.io/remediation-strategy"
 	remediationStrategyExternal   = mapiv1.RemediationStrategyType("external-baremetal")
-	timeoutForMachineToHaveNode   = 10 * time.Minute
+	defaultNodeStartupTimeout     = 10 * time.Minute
 	machineNodeNameIndex          = "machineNodeNameIndex"
 	controllerName                = "machinehealthcheck-controller"
 
@@ -51,10 +51,9 @@ const (
 	// EventDetectedUnhealthy is emitted in case a node asociated with a
 	// machine was detected unhealthy
 	EventDetectedUnhealthy string = "DetectedUnhealthy"
-	// EventSkippedMaster is emitted in case an unhealthy node (or a machine
-	// associated with the node) has Master role and external remediation strategy
-	// is not enabled
-	EventSkippedMaster string = "SkippedMaster"
+	// EventSkippedNoController is emitted in case an unhealthy node (or a machine
+	// associated with the node) has no controller owner
+	EventSkippedNoController string = "SkippedNoController"
 	// EventMachineDeletionFailed is emitted in case remediation of a machine
 	// is required but deletion of its Machine object failed
 	EventMachineDeletionFailed string = "MachineDeletionFailed"
@@ -81,7 +80,8 @@ func Add(mgr manager.Manager, opts manager.Options) error {
 
 // newReconciler returns a new reconcile.Reconciler
 func newReconciler(mgr manager.Manager, opts manager.Options) (*ReconcileMachineHealthCheck, error) {
-	if err := mgr.GetCache().IndexField(&mapiv1.Machine{},
+	if err := mgr.GetCache().IndexField(context.TODO(),
+		&mapiv1.Machine{},
 		machineNodeNameIndex,
 		indexMachineByNodeName,
 	); err != nil {
@@ -96,10 +96,10 @@ func newReconciler(mgr manager.Manager, opts manager.Options) (*ReconcileMachine
 	}, nil
 }
 
-func indexMachineByNodeName(object runtime.Object) []string {
+func indexMachineByNodeName(object client.Object) []string {
 	machine, ok := object.(*mapiv1.Machine)
 	if !ok {
-		glog.Warningf("Expected a machine for indexing field, got: %T", object)
+		klog.Warningf("Expected a machine for indexing field, got: %T", object)
 		return nil
 	}
 
@@ -111,7 +111,7 @@ func indexMachineByNodeName(object runtime.Object) []string {
 }
 
 // add adds a new Controller to mgr with r as the reconcile.Reconciler
-func add(mgr manager.Manager, r reconcile.Reconciler, mapMachineToMHC, mapNodeToMHC handler.ToRequestsFunc) error {
+func add(mgr manager.Manager, r reconcile.Reconciler, mapMachineToMHC, mapNodeToMHC handler.MapFunc) error {
 	c, err := controller.New(controllerName, mgr, controller.Options{Reconciler: r})
 	if err != nil {
 		return err
@@ -122,12 +122,12 @@ func add(mgr manager.Manager, r reconcile.Reconciler, mapMachineToMHC, mapNodeTo
 		return err
 	}
 
-	err = c.Watch(&source.Kind{Type: &mapiv1.Machine{}}, &handler.EnqueueRequestsFromMapFunc{ToRequests: mapMachineToMHC})
+	err = c.Watch(&source.Kind{Type: &mapiv1.Machine{}}, handler.EnqueueRequestsFromMapFunc(mapMachineToMHC))
 	if err != nil {
 		return err
 	}
 
-	return c.Watch(&source.Kind{Type: &corev1.Node{}}, &handler.EnqueueRequestsFromMapFunc{ToRequests: mapNodeToMHC})
+	return c.Watch(&source.Kind{Type: &corev1.Node{}}, handler.EnqueueRequestsFromMapFunc(mapNodeToMHC))
 }
 
 var _ reconcile.Reconciler = &ReconcileMachineHealthCheck{}
@@ -149,65 +149,101 @@ type target struct {
 }
 
 // Reconcile fetch all targets for a MachineHealthCheck request and does health checking for each of them
-func (r *ReconcileMachineHealthCheck) Reconcile(request reconcile.Request) (reconcile.Result, error) {
-	glog.Infof("Reconciling %s", request.String())
+func (r *ReconcileMachineHealthCheck) Reconcile(ctx context.Context, request reconcile.Request) (reconcile.Result, error) {
+	klog.Infof("Reconciling %s", request.String())
 
 	mhc := &mapiv1.MachineHealthCheck{}
 	if err := r.client.Get(context.TODO(), request.NamespacedName, mhc); err != nil {
 		if apimachineryerrors.IsNotFound(err) {
 			// Request object not found, could have been deleted after reconcile request.
+			// In the event that this was a deletion, we need to remove the associated metric label
+			metrics.DeleteMachineHealthCheckNodesCovered(request.NamespacedName.Name, request.NamespacedName.Namespace)
 			return reconcile.Result{}, nil
 		}
-		glog.Errorf("Reconciling %s: failed to get MHC: %v", request.String(), err)
+		klog.Errorf("Reconciling %s: failed to get MHC: %v", request.String(), err)
 		return reconcile.Result{}, err
 	}
 
+	// Create a base from which the MHC status patch will be calculated
+	mergeBase := client.MergeFrom(mhc.DeepCopy())
+
 	// fetch all targets
-	glog.V(3).Infof("Reconciling %s: finding targets", request.String())
+	klog.V(3).Infof("Reconciling %s: finding targets", request.String())
 	targets, err := r.getTargetsFromMHC(*mhc)
 	if err != nil {
 		return reconcile.Result{}, err
 	}
 	totalTargets := len(targets)
 
+	metrics.ObserveMachineHealthCheckNodesCovered(mhc.Name, mhc.Namespace, totalTargets)
+
 	// health check all targets and reconcile mhc status
-	currentHealthy, needRemediationTargets, nextCheckTimes, errList := r.healthCheckTargets(targets)
-	if err := r.reconcileStatus(mhc, totalTargets, currentHealthy); err != nil {
-		glog.Errorf("Reconciling %s: error patching status: %v", request.String(), err)
-		return reconcile.Result{}, err
-	}
+	currentHealthy, needRemediationTargets, nextCheckTimes, errList := r.healthCheckTargets(targets, mhc.Spec.NodeStartupTimeout.Duration)
+	mhc.Status.CurrentHealthy = &currentHealthy
+	mhc.Status.ExpectedMachines = &totalTargets
+	unhealthyCount := totalTargets - currentHealthy
 
 	// check MHC current health against MaxUnhealthy
 	if !isAllowedRemediation(mhc) {
-		glog.Warningf("Reconciling %s: total targets: %v,  maxUnhealthy: %v, unhealthy: %v. Short-circuiting remediation",
+		klog.Warningf("Reconciling %s: total targets: %v,  maxUnhealthy: %v, unhealthy: %v. Short-circuiting remediation",
 			request.String(),
 			totalTargets,
 			mhc.Spec.MaxUnhealthy,
 			totalTargets-currentHealthy,
 		)
+
+		message := fmt.Sprintf("Remediation is not allowed, the number of not started or unhealthy machines exceeds maxUnhealthy (total: %v, unhealthy: %v, maxUnhealthy: %v)",
+			totalTargets,
+			unhealthyCount,
+			mhc.Spec.MaxUnhealthy,
+		)
+
+		// Remediation not allowed, the number of not started or unhealthy machines exceeds maxUnhealthy
+		mhc.Status.RemediationsAllowed = 0
+		conditions.Set(mhc, &mapiv1.Condition{
+			Type:     mapiv1.RemediationAllowedCondition,
+			Status:   corev1.ConditionFalse,
+			Severity: mapiv1.ConditionSeverityWarning,
+			Reason:   mapiv1.TooManyUnhealthyReason,
+			Message:  message,
+		})
+
+		if err := r.reconcileStatus(mergeBase, mhc); err != nil {
+			klog.Errorf("Reconciling %s: error patching status: %v", request.String(), err)
+			return reconcile.Result{}, err
+		}
+
 		r.recorder.Eventf(
 			mhc,
 			corev1.EventTypeWarning,
 			EventRemediationRestricted,
 			"Remediation restricted due to exceeded number of unhealthy machines (total: %v, unhealthy: %v, maxUnhealthy: %v)",
 			totalTargets,
-			totalTargets-currentHealthy,
+			unhealthyCount,
 			mhc.Spec.MaxUnhealthy,
 		)
+		metrics.ObserveMachineHealthCheckShortCircuitEnabled(mhc.Name, mhc.Namespace)
 		return reconcile.Result{Requeue: true}, nil
 	}
-	glog.V(3).Infof("Reconciling %s: monitoring MHC: total targets: %v,  maxUnhealthy: %v, unhealthy: %v. Remediations are allowed",
+	klog.V(3).Infof("Remediations are allowed for %s: total targets: %v,  max unhealthy: %v, unhealthy targets: %v",
 		request.String(),
 		totalTargets,
 		mhc.Spec.MaxUnhealthy,
-		totalTargets-currentHealthy,
+		unhealthyCount,
 	)
+	metrics.ObserveMachineHealthCheckShortCircuitDisabled(mhc.Name, mhc.Namespace)
+
+	conditions.MarkTrue(mhc, mapiv1.RemediationAllowedCondition)
+	if err := r.reconcileStatus(mergeBase, mhc); err != nil {
+		klog.Errorf("Reconciling %s: error patching status: %v", request.String(), err)
+		return reconcile.Result{}, err
+	}
 
 	// remediate
 	for _, t := range needRemediationTargets {
-		glog.V(3).Infof("Reconciling %s: meet unhealthy criteria, triggers remediation", t.string())
+		klog.V(3).Infof("Reconciling %s: meet unhealthy criteria, triggers remediation", t.string())
 		if err := t.remediate(r); err != nil {
-			glog.Errorf("Reconciling %s: error remediating: %v", t.string(), err)
+			klog.Errorf("Reconciling %s: error remediating: %v", t.string(), err)
 			errList = append(errList, err)
 		}
 	}
@@ -215,32 +251,51 @@ func (r *ReconcileMachineHealthCheck) Reconcile(request reconcile.Request) (reco
 	// return values
 	if len(errList) > 0 {
 		requeueError := apimachineryutilerrors.NewAggregate(errList)
-		glog.V(3).Infof("Reconciling %s: there were errors, requeuing: %v", request.String(), requeueError)
+		klog.V(3).Infof("Reconciling %s: there were errors, requeuing: %v", request.String(), requeueError)
 		return reconcile.Result{}, requeueError
 	}
 
 	if minNextCheck := minDuration(nextCheckTimes); minNextCheck > 0 {
-		glog.V(3).Infof("Reconciling %s: some targets might go unhealthy. Ensuring a requeue happens in %v", request.String(), minNextCheck)
+		klog.V(3).Infof("Reconciling %s: some targets might go unhealthy. Ensuring a requeue happens in %v", request.String(), minNextCheck)
 		return reconcile.Result{RequeueAfter: minNextCheck}, nil
 	}
 
-	glog.V(3).Infof("Reconciling %s: no more targets meet unhealthy criteria", request.String())
+	klog.V(3).Infof("Reconciling %s: no more targets meet unhealthy criteria", request.String())
 	return reconcile.Result{}, nil
 }
 
 func isAllowedRemediation(mhc *mapiv1.MachineHealthCheck) bool {
-	if mhc.Spec.MaxUnhealthy == nil {
-		return true
-	}
-	maxUnhealthy, err := getValueFromIntOrPercent(mhc.Spec.MaxUnhealthy, derefInt(mhc.Status.ExpectedMachines), false)
+	maxUnhealthy, err := getMaxUnhealthy(mhc)
 	if err != nil {
-		glog.Errorf("%s: error decoding maxUnhealthy, remediation won't be allowed: %v", namespacedName(mhc), err)
 		return false
 	}
 
-	// if noHealthy are above maxUnhealthy we short circuit any farther remediation
-	noHealthy := derefInt(mhc.Status.ExpectedMachines) - derefInt(mhc.Status.CurrentHealthy)
-	return (maxUnhealthy - noHealthy) >= 0
+	// If unhealthy is above maxUnhealthy, short circuit any further remediation
+	return unhealthyMachineCount(mhc) <= maxUnhealthy
+}
+
+func getMaxUnhealthy(mhc *mapiv1.MachineHealthCheck) (int, error) {
+	if mhc.Spec.MaxUnhealthy == nil {
+		// This value should be defaulted, but if not, 100% is the default
+		return derefInt(mhc.Status.ExpectedMachines), nil
+	}
+	maxUnhealthy, err := getValueFromIntOrPercent(mhc.Spec.MaxUnhealthy, derefInt(mhc.Status.ExpectedMachines), false)
+	if err != nil {
+		klog.Errorf("%s: error decoding maxUnhealthy, remediation won't be allowed: %v", namespacedName(mhc), err)
+		return 0, err
+	}
+
+	if maxUnhealthy < 0 {
+		maxUnhealthy = 0
+	}
+
+	return maxUnhealthy, nil
+}
+
+// unhealthyMachineCount calculates the number of presently unhealthy or missing machines
+// ie the delta between the expected number of machines and the current number deemed healthy
+func unhealthyMachineCount(mhc *mapiv1.MachineHealthCheck) int {
+	return derefInt(mhc.Status.ExpectedMachines) - derefInt(mhc.Status.CurrentHealthy)
 }
 
 func derefInt(i *int) int {
@@ -250,10 +305,15 @@ func derefInt(i *int) int {
 	return 0
 }
 
-func (r *ReconcileMachineHealthCheck) reconcileStatus(mhc *mapiv1.MachineHealthCheck, targets, currentHealthy int) error {
-	baseToPatch := client.MergeFrom(mhc.DeepCopy())
-	mhc.Status.ExpectedMachines = &targets
-	mhc.Status.CurrentHealthy = &currentHealthy
+func (r *ReconcileMachineHealthCheck) reconcileStatus(baseToPatch client.Patch, mhc *mapiv1.MachineHealthCheck) error {
+	maxUnhealthy, err := getMaxUnhealthy(mhc)
+	if err != nil {
+		return fmt.Errorf("failed to get value for maxUnhealthy: %v", err)
+	}
+	mhc.Status.RemediationsAllowed = int32(maxUnhealthy - unhealthyMachineCount(mhc))
+	if mhc.Status.RemediationsAllowed < 0 {
+		mhc.Status.RemediationsAllowed = 0
+	}
 
 	if err := r.client.Status().Patch(context.Background(), mhc, baseToPatch); err != nil {
 		return err
@@ -263,16 +323,16 @@ func (r *ReconcileMachineHealthCheck) reconcileStatus(mhc *mapiv1.MachineHealthC
 
 // healthCheckTargets health checks a slice of targets
 // and gives a data to measure the average health
-func (r *ReconcileMachineHealthCheck) healthCheckTargets(targets []target) (int, []target, []time.Duration, []error) {
+func (r *ReconcileMachineHealthCheck) healthCheckTargets(targets []target, timeoutForMachineToHaveNode time.Duration) (int, []target, []time.Duration, []error) {
 	var nextCheckTimes []time.Duration
 	var errList []error
 	var needRemediationTargets []target
 	var currentHealthy int
 	for _, t := range targets {
-		glog.V(3).Infof("Reconciling %s: health checking", t.string())
-		needsRemediation, nextCheck, err := t.needsRemediation()
+		klog.V(3).Infof("Reconciling %s: health checking", t.string())
+		needsRemediation, nextCheck, err := t.needsRemediation(timeoutForMachineToHaveNode)
 		if err != nil {
-			glog.Errorf("Reconciling %s: error health checking: %v", t.string(), err)
+			klog.Errorf("Reconciling %s: error health checking: %v", t.string(), err)
 			errList = append(errList, err)
 			continue
 		}
@@ -283,7 +343,7 @@ func (r *ReconcileMachineHealthCheck) healthCheckTargets(targets []target) (int,
 		}
 
 		if nextCheck > 0 {
-			glog.V(3).Infof("Reconciling %s: is likely to go unhealthy in %v", t.string(), nextCheck)
+			klog.V(3).Infof("Reconciling %s: is likely to go unhealthy in %v", t.string(), nextCheck)
 			r.recorder.Eventf(
 				&t.Machine,
 				corev1.EventTypeNormal,
@@ -365,27 +425,27 @@ func (r *ReconcileMachineHealthCheck) getMachineFromNode(nodeName string) (*mapi
 	return &machineList.Items[0], nil
 }
 
-func (r *ReconcileMachineHealthCheck) mhcRequestsFromNode(o handler.MapObject) []reconcile.Request {
-	glog.V(4).Infof("Getting MHC requests from node %q", namespacedName(o.Meta).String())
+func (r *ReconcileMachineHealthCheck) mhcRequestsFromNode(o client.Object) []reconcile.Request {
+	klog.V(4).Infof("Getting MHC requests from node %q", namespacedName(o).String())
 	node := &corev1.Node{}
-	if err := r.client.Get(context.Background(), namespacedName(o.Meta), node); err != nil {
+	if err := r.client.Get(context.Background(), namespacedName(o), node); err != nil {
 		if apimachineryerrors.IsNotFound(err) {
-			node.Name = o.Meta.GetName()
+			node.Name = o.GetName()
 		} else {
-			glog.Errorf("No-op: Unable to retrieve node %q from store: %v", namespacedName(o.Meta).String(), err)
+			klog.Errorf("No-op: Unable to retrieve node %q from store: %v", namespacedName(o).String(), err)
 			return nil
 		}
 	}
 
 	machine, err := r.getMachineFromNode(node.Name)
 	if machine == nil || err != nil {
-		glog.Errorf("No-op: Unable to retrieve machine from node %q: %v", namespacedName(node).String(), err)
+		klog.Errorf("No-op: Unable to retrieve machine from node %q: %v", namespacedName(node).String(), err)
 		return nil
 	}
 
 	mhcList := &mapiv1.MachineHealthCheckList{}
 	if err := r.client.List(context.Background(), mhcList); err != nil {
-		glog.Errorf("No-op: Unable to list mhc: %v", err)
+		klog.Errorf("No-op: Unable to list mhc: %v", err)
 		return nil
 	}
 
@@ -399,23 +459,23 @@ func (r *ReconcileMachineHealthCheck) mhcRequestsFromNode(o handler.MapObject) [
 	return requests
 }
 
-func (r *ReconcileMachineHealthCheck) mhcRequestsFromMachine(o handler.MapObject) []reconcile.Request {
-	glog.V(4).Infof("Getting MHC requests from machine %q", namespacedName(o.Meta).String())
+func (r *ReconcileMachineHealthCheck) mhcRequestsFromMachine(o client.Object) []reconcile.Request {
+	klog.V(4).Infof("Getting MHC requests from machine %q", namespacedName(o).String())
 	machine := &mapiv1.Machine{}
 	if err := r.client.Get(context.Background(),
 		client.ObjectKey{
-			Namespace: o.Meta.GetNamespace(),
-			Name:      o.Meta.GetName(),
+			Namespace: o.GetNamespace(),
+			Name:      o.GetName(),
 		},
 		machine,
 	); err != nil {
-		glog.Errorf("No-op: Unable to retrieve machine %q from store: %v", namespacedName(o.Meta).String(), err)
+		klog.Errorf("No-op: Unable to retrieve machine %q from store: %v", namespacedName(o).String(), err)
 		return nil
 	}
 
 	mhcList := &mapiv1.MachineHealthCheckList{}
 	if err := r.client.List(context.Background(), mhcList); err != nil {
-		glog.Errorf("No-op: Unable to list mhc: %v", err)
+		klog.Errorf("No-op: Unable to list mhc: %v", err)
 		return nil
 	}
 
@@ -429,31 +489,44 @@ func (r *ReconcileMachineHealthCheck) mhcRequestsFromMachine(o handler.MapObject
 }
 
 func (t *target) remediate(r *ReconcileMachineHealthCheck) error {
-	glog.Infof(" %s: start remediation logic", t.string())
-	if !t.hasMachineSetOwner() {
-		glog.Infof("%s: no machineSet controller owner, skipping remediation", t.string())
-		return nil
-	}
+	klog.Infof(" %s: start remediation logic", t.string())
 
-	remediationStrategy, ok := t.MHC.Annotations[remediationStrategyAnnotation]
-	if ok {
-		if mapiv1.RemediationStrategyType(remediationStrategy) == remediationStrategyExternal {
-			return t.remediationStrategyExternal(r)
+	if derefStringPointer(t.Machine.Status.Phase) != machinePhaseFailed {
+		if remediationStrategy, ok := t.MHC.Annotations[remediationStrategyAnnotation]; ok {
+			if mapiv1.RemediationStrategyType(remediationStrategy) == remediationStrategyExternal {
+				return t.remediationStrategyExternal(r)
+			}
 		}
 	}
-	if t.isMaster() {
+
+	if !t.hasControllerOwner() {
 		r.recorder.Eventf(
 			&t.Machine,
 			corev1.EventTypeNormal,
-			EventSkippedMaster,
-			"Machine %v is a master node, skipping remediation",
+			EventSkippedNoController,
+			"Machine %v has no controller owner, skipping remediation",
 			t.string(),
 		)
-		glog.Infof("%s: master node, skipping remediation", t.string())
+		klog.Infof("%s: no controller owner, skipping remediation", t.string())
 		return nil
 	}
 
-	glog.Infof("%s: deleting", t.string())
+	key := client.ObjectKey{Namespace: t.Machine.Namespace, Name: t.Machine.Name}
+	machine := &mapiv1.Machine{}
+	if err := r.client.Get(context.TODO(), key, machine); err != nil {
+		if apimachineryerrors.IsNotFound(err) {
+			// Machine has already been deleted
+			return nil
+		}
+		return fmt.Errorf("%s: failed to get machine: %v", t.string(), err)
+	}
+
+	if !machine.GetDeletionTimestamp().IsZero() {
+		// Delete already initiated
+		return nil
+	}
+
+	klog.Infof("%s: deleting", t.string())
 	if err := r.client.Delete(context.TODO(), &t.Machine); err != nil {
 		r.recorder.Eventf(
 			&t.Machine,
@@ -472,6 +545,8 @@ func (t *target) remediate(r *ReconcileMachineHealthCheck) error {
 		"Machine %v has been remediated by requesting to delete Machine object",
 		t.string(),
 	)
+	metrics.ObserveMachineHealthCheckRemediationSuccess(t.MHC.Name, t.MHC.Namespace)
+
 	return nil
 }
 
@@ -485,7 +560,7 @@ func (t *target) remediationStrategyExternal(r *ReconcileMachineHealthCheck) err
 		t.Machine.Annotations = map[string]string{}
 	}
 
-	glog.Infof("Machine %s has been unhealthy for too long, adding external annotation", t.Machine.Name)
+	klog.Infof("Machine %s has been unhealthy for too long, adding external annotation", t.Machine.Name)
 	t.Machine.Annotations[machineExternalAnnotationKey] = ""
 	if err := r.client.Update(context.TODO(), &t.Machine); err != nil {
 		r.recorder.Eventf(
@@ -538,28 +613,13 @@ func (t *target) nodeName() string {
 	return ""
 }
 
-func (t *target) isMaster() bool {
-	if t.Node != nil {
-		if labels.Set(t.Node.Labels).Has(nodeMasterLabel) {
-			return true
-		}
-	}
-
-	// if the node is not found we fallback to check the machine
-	if labels.Set(t.Machine.Labels).Get(machineRoleLabel) == machineMasterRole {
-		return true
-	}
-
-	return false
-}
-
-func (t *target) needsRemediation() (bool, time.Duration, error) {
+func (t *target) needsRemediation(timeoutForMachineToHaveNode time.Duration) (bool, time.Duration, error) {
 	var nextCheckTimes []time.Duration
 	now := time.Now()
 
 	// machine has failed
 	if derefStringPointer(t.Machine.Status.Phase) == machinePhaseFailed {
-		glog.V(3).Infof("%s: unhealthy: machine phase is %q", t.string(), machinePhaseFailed)
+		klog.V(3).Infof("%s: unhealthy: machine phase is %q", t.string(), machinePhaseFailed)
 		return true, time.Duration(0), nil
 	}
 
@@ -570,7 +630,7 @@ func (t *target) needsRemediation() (bool, time.Duration, error) {
 			return false, timeoutForMachineToHaveNode, nil
 		}
 		if t.Machine.Status.LastUpdated.Add(timeoutForMachineToHaveNode).Before(now) {
-			glog.V(3).Infof("%s: unhealthy: machine has no node after %v", t.string(), timeoutForMachineToHaveNode)
+			klog.V(3).Infof("%s: unhealthy: machine has no node after %v", t.string(), timeoutForMachineToHaveNode)
 			return true, time.Duration(0), nil
 		}
 		durationUnhealthy := now.Sub(t.Machine.Status.LastUpdated.Time)
@@ -588,11 +648,6 @@ func (t *target) needsRemediation() (bool, time.Duration, error) {
 		now := time.Now()
 		nodeCondition := conditions.GetNodeCondition(t.Node, c.Type)
 
-		timeout, err := time.ParseDuration(c.Timeout)
-		if err != nil {
-			return false, time.Duration(0), fmt.Errorf("error parsing duration: %v", err)
-		}
-
 		// Skip when current node condition is different from the one reported
 		// in the MachineHealthCheck.
 		if nodeCondition == nil || nodeCondition.Status != c.Status {
@@ -601,13 +656,13 @@ func (t *target) needsRemediation() (bool, time.Duration, error) {
 
 		// If the condition has been in the unhealthy state for longer than the
 		// timeout, return true with no requeue time.
-		if nodeCondition.LastTransitionTime.Add(timeout).Before(now) {
-			glog.V(3).Infof("%s: unhealthy: condition %v in state %v longer than %v", t.string(), c.Type, c.Status, c.Timeout)
+		if nodeCondition.LastTransitionTime.Add(c.Timeout.Duration).Before(now) {
+			klog.V(3).Infof("%s: unhealthy: condition %v in state %v longer than %v", t.string(), c.Type, c.Status, c.Timeout)
 			return true, time.Duration(0), nil
 		}
 
 		durationUnhealthy := now.Sub(nodeCondition.LastTransitionTime.Time)
-		nextCheck := timeout - durationUnhealthy + time.Second
+		nextCheck := c.Timeout.Duration - durationUnhealthy + time.Second
 		if nextCheck > 0 {
 			nextCheckTimes = append(nextCheckTimes, nextCheck)
 		}
@@ -615,14 +670,8 @@ func (t *target) needsRemediation() (bool, time.Duration, error) {
 	return false, minDuration(nextCheckTimes), nil
 }
 
-func (t *target) hasMachineSetOwner() bool {
-	ownerRefs := t.Machine.ObjectMeta.GetOwnerReferences()
-	for _, or := range ownerRefs {
-		if or.Kind == ownerControllerKind {
-			return true
-		}
-	}
-	return false
+func (t *target) hasControllerOwner() bool {
+	return metav1.GetControllerOf(&t.Machine) != nil
 }
 
 func derefStringPointer(stringPointer *string) string {
@@ -632,13 +681,6 @@ func derefStringPointer(stringPointer *string) string {
 	return ""
 }
 
-func derefStringInt(intPointer *int) int {
-	if intPointer != nil {
-		return 0
-	}
-	return *intPointer
-}
-
 func minDuration(durations []time.Duration) time.Duration {
 	if len(durations) == 0 {
 		return time.Duration(0)
@@ -663,16 +705,15 @@ func namespacedName(obj metav1.Object) types.NamespacedName {
 func hasMatchingLabels(machineHealthCheck *mapiv1.MachineHealthCheck, machine *mapiv1.Machine) bool {
 	selector, err := metav1.LabelSelectorAsSelector(&machineHealthCheck.Spec.Selector)
 	if err != nil {
-		glog.Warningf("unable to convert selector: %v", err)
+		klog.Warningf("unable to convert selector: %v", err)
 		return false
 	}
-	// If a deployment with a nil or empty selector creeps in, it should match nothing, not everything.
+	// If the selector is empty, all machines are considered to match
 	if selector.Empty() {
-		glog.V(3).Infof("%q machineHealthCheck has empty selector", machineHealthCheck.GetName())
-		return false
+		return true
 	}
 	if !selector.Matches(labels.Set(machine.Labels)) {
-		glog.V(4).Infof("%q machine has mismatched labels for MHC %q", machine.GetName(), machineHealthCheck.GetName())
+		klog.V(4).Infof("%q machine has mismatched labels for MHC %q", machine.GetName(), machineHealthCheck.GetName())
 		return false
 	}
 	return true
@@ -715,9 +756,9 @@ func getIntOrPercentValue(intOrStr *intstr.IntOrString) (int, bool, error) {
 	case intstr.String:
 		isPercent := false
 		s := intOrStr.StrVal
-		if strings.Contains(s, "%") {
+		if strings.HasSuffix(s, "%") {
 			isPercent = true
-			s = strings.Replace(intOrStr.StrVal, "%", "", -1)
+			s = strings.TrimSuffix(intOrStr.StrVal, "%")
 		}
 		v, err := strconv.Atoi(s)
 		if err != nil {
